/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.example.endpoint.hl7v2.inbound;

import ca.cdr.api.fhir.interceptor.CdrHook;
import ca.cdr.api.fhir.interceptor.CdrPointcut;
import ca.cdr.api.pub.hl7v2.model.Hl7v2ToFhirConversionResultJson;
import ca.uhn.fhir.context.FhirContext;
import ca.uhn.fhir.interceptor.api.Interceptor;
import ca.uhn.fhir.util.BundleUtil;
import org.hl7.fhir.instance.model.api.IBaseBundle;
import org.hl7.fhir.r4.model.Patient;
import org.hl7.fhir.r4.model.StringType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

@Interceptor
public class PostConvertInterceptor {

	private static final Logger ourLog = LoggerFactory.getLogger(PostConvertInterceptor.class);

	public static final String EXTENSION_SYSTEM = "http://acme.com/some-system";
	public static final String EXTENSION_VALUE = "abc123";

	private final FhirContext myFhirContext;

	public PostConvertInterceptor() {
		myFhirContext = FhirContext.forR4Cached();
	}

	/**
	 * Interceptor that will be called after the Hl7v2 Message is converted into a FHIR bundle.
	 * <br/><br/>
	 * <strong>NOTE:</strong> {@link Hl7v2ToFhirConversionResultJson} is the only parameter that should be used with
	 * this pointcut. See {@link CdrPointcut#HL7V2IN_POST_HL7V2_TO_FHIR_MAPPING_PROCESSING} for more details.
	 *
	 * @param theConversionResult  	relevant data for the Hl7v2 to FHIR conversion
	 */
	@CdrHook(CdrPointcut.HL7V2IN_POST_HL7V2_TO_FHIR_MAPPING_PROCESSING)
	public void hl7v2ToFhirPostProcess(Hl7v2ToFhirConversionResultJson theConversionResult) {
		try {
			ourLog.info("Interceptor HL7V2IN_POST_HL7V2_TO_FHIR_MAPPING_PROCESSING - started");

			// get the first Bundle that was converted from the hl7v2 Message
			IBaseBundle bundle = theConversionResult.getBundles().get(0);

			// get a list of all Patients that were added to the bundle
			List<Patient> patients = BundleUtil.toListOfResourcesOfType(myFhirContext, bundle, Patient.class);

			// add a custom extension to each Patient
			for (Patient patient : patients) {
				patient.addExtension().setUrl(EXTENSION_SYSTEM).setValue(new StringType(EXTENSION_VALUE));
			}
		} finally {
			ourLog.info("Interceptor HL7V2IN_POST_HL7V2_TO_FHIR_MAPPING_PROCESSING - ended");
		}
	}

	public FhirContext getFhirContext() {
		return myFhirContext;
	}
}
