/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.example.endpoint.hl7v2.inbound;

import ca.cdr.api.fhir.interceptor.CdrHook;
import ca.cdr.api.fhir.interceptor.CdrPointcut;
import ca.cdr.api.pub.hl7v2.model.Hl7v2ToFhirConversionResultJson;
import ca.uhn.fhir.interceptor.api.Interceptor;
import ca.uhn.hl7v2.HL7Exception;
import ca.uhn.hl7v2.model.Message;
import ca.uhn.hl7v2.model.v25.datatype.CX;
import ca.uhn.hl7v2.model.v25.segment.PID;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Interceptor
public class PreConvertInterceptor {

	private static final Logger ourLog = LoggerFactory.getLogger(PreConvertInterceptor.class);

	public static final String SECONDARY_IDENTIFIER_SYSTEM = "http://acme.org/secondaryIds";
	public static final String SECONDARY_IDENTIFIER_CODE = "SB";

	/**
	 * Interceptor that will be called before the Hl7v2 Message is converted into a FHIR bundle.
	 * <br/><br/>
	 * <strong>NOTE:</strong> {@link Hl7v2ToFhirConversionResultJson} is the only parameter that should be used with
	 * this pointcut. See {@link CdrPointcut#HL7V2IN_PRE_HL7V2_TO_FHIR_MAPPING_PROCESSING} for more details.
	 *
	 * @param theConversionResult  		relevant data for the Hl7v2 to FHIR conversion
	 * @return 							true if the Message should continue being processed
	 * @throws HL7Exception				if there is an error during processing
	 */
	@CdrHook(CdrPointcut.HL7V2IN_PRE_HL7V2_TO_FHIR_MAPPING_PROCESSING)
	public Boolean hl7v2ToFhirPreProcess(Hl7v2ToFhirConversionResultJson theConversionResult) throws HL7Exception {
		try {
			ourLog.info("Interceptor HL7V2IN_PRE_HL7V2_TO_FHIR_MAPPING_PROCESSING - started");

			// obtain the Message that can be modified
			Message modifiableMessage = theConversionResult.getModifiableMessage();

			// obtain the PID segment
			PID pid = (PID) modifiableMessage.get("PID");

			if (pid.getPid3_PatientIdentifierListReps() == 2) {
				ourLog.info("Patient already has a secondary identifier. A second identifier will not be added.");
			} else {
				addSecondaryIdentifier(pid);
			}
		} finally {
			ourLog.info("Interceptor HL7V2IN_PRE_HL7V2_TO_FHIR_MAPPING_PROCESSING - ended");
		}

		// continue processing
		return true;
	}

	private void addSecondaryIdentifier(PID pid) throws HL7Exception {
		String primaryIdentifierValue =
				pid.getPid3_PatientIdentifierList(0).getCx1_IDNumber().getValue();
		CX secondaryIdentifier = pid.insertPid3_PatientIdentifierList(1);
		secondaryIdentifier.getCx1_IDNumber().setValue(primaryIdentifierValue);
		secondaryIdentifier.getCx4_AssigningAuthority().getHd1_NamespaceID().setValue(SECONDARY_IDENTIFIER_SYSTEM);
		secondaryIdentifier.getCx5_IdentifierTypeCode().setValue(SECONDARY_IDENTIFIER_CODE);
	}
}
